<?php

require_once _PS_MODULE_DIR_ . 'billease/lib/BillEaseAutoload.php';

class BillEase extends PaymentModule
{
    public function __construct()
    {
        $this->name = 'billease';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.9';
        $this->controllers = array('payment', 'validation');
        $this->author = 'FDFC';

        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        parent::__construct();

        \BillEase\Settings::$merchantCode = trim(Configuration::get('BILLEASE_MERCHANT_CODE'));
        \BillEase\Settings::$shopCode = trim(Configuration::get('BILLEASE_SHOP_CODE'));
        \BillEase\Settings::$merchantJwt = trim(Configuration::get('BILLEASE_MERCHANT_JWT'));
        \BillEase\Settings::$testMode = trim(Configuration::get('BILLEASE_TEST_MODE'));
        \BillEase\Settings::$paymentMethodDescription = trim(Configuration::get('BILLEASE_PAYMENT_METHOD_DESCRIPTION'));

        $this->page = basename(__FILE__, '.php');
        $this->displayName = $this->l('BillEase');
        $this->description = $this->l('BillEase | Buy Now, Pay Later. No card required.');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
    }

    public function install()
    {
        if (!parent::install()
            OR !Configuration::updateValue('BILLEASE_MERCHANT_CODE', '')
            OR !Configuration::updateValue('BILLEASE_SHOP_CODE', '')
            OR !Configuration::updateValue('BILLEASE_MERCHANT_JWT', '')
            OR !Configuration::updateValue('BILLEASE_TEST_MODE', 1)
            OR !$this->registerHook('payment')
            OR !$this->registerHook('adminOrder')
            OR !$this->createOrderState()) {
            return false;
        }

        if (_PS_VERSION_ > 1.4 && !$this->registerHook('displayHeader')) {
            return false;
        }

        return true;
    }

    public function uninstall()
    {
        if (!Configuration::deleteByName('BILLEASE_MERCHANT_CODE')
            OR !Configuration::deleteByName('BILLEASE_SHOP_CODE')
            OR !Configuration::deleteByName('BILLEASE_MERCHANT_JWT')
            OR !Configuration::deleteByName('BILLEASE_TEST_MODE')
            OR !Configuration::deleteByName('BILLEASE_PAYMENT_METHOD_DESCRIPTION')
            OR !$this->unregisterHook('payment')
            OR !$this->unregisterHook('displayHeader')
            OR !parent::uninstall())
            return false;
        return true;
    }

    public function getContent()
    {
        if (Tools::isSubmit('submit' . $this->name)) {
            $this->updateConfigurationPost();
        }

        $data = [
            'base_url' => _PS_BASE_URL_SSL_ . __PS_BASE_URI__,
            'module_name' => $this->name,
            'form' => $this->displayForm(),
        ];

        $this->context->smarty->assign($data);
        $output = $this->display(__FILE__, 'views/templates/admin/configuration.tpl');

        return $output;
    }

    public function hookdisplayHeader()
    {
        if (!$this->active)
            return;

        $this->context->controller->addCSS($this->getPathUri() . 'views/css/billease.css');

        if (_PS_VERSION_ < 1.6)
            $this->context->controller->addCSS($this->getPathUri() . 'views/css/billease_1_5.css');
    }

    public function hookPayment($params)
    {
        if (!$this->active)
            return;

        $this->context->smarty->assign(
            [
                'billease_path' => $this->getPathUri(),
                'controller_link' => BillEaseHelper::controllerLink('payment'),
                'billease_payment_description' => \BillEase\Settings::$paymentMethodDescription
            ]
        );

        return $this->display(__FILE__, 'views/templates/hook/payment.tpl');
    }

    protected function updateConfigurationPost()
    {
        Configuration::updateValue('BILLEASE_MERCHANT_CODE', trim(Tools::getValue('BILLEASE_MERCHANT_CODE')));
        Configuration::updateValue('BILLEASE_SHOP_CODE', trim(Tools::getValue('BILLEASE_SHOP_CODE')));
        Configuration::updateValue('BILLEASE_MERCHANT_JWT', trim(Tools::getValue('BILLEASE_MERCHANT_JWT')));
        Configuration::updateValue('BILLEASE_TEST_MODE', trim(Tools::getValue('BILLEASE_TEST_MODE')));
        Configuration::updateValue('BILLEASE_PAYMENT_METHOD_DESCRIPTION', trim(Tools::getValue('BILLEASE_PAYMENT_METHOD_DESCRIPTION')));
    }

    protected function displayForm()
    {
        $fieldsForm = [];

        $defaultLang = (int)Configuration::get('PS_LANG_DEFAULT');

        $fieldsForm[0]['form'] = [
            'legend' => [
                'title' => $this->l('Configuration'),
                'icon' => 'icon-cogs',
            ],
            'input' => [
                [
                    'type' => 'text',
                    'label' => $this->l('Merchant code'),
                    'name' => 'BILLEASE_MERCHANT_CODE',
                    'size' => 40,
                    'required' => true,
                ],
                [
                    'type' => 'text',
                    'label' => $this->l('Shop code'),
                    'name' => 'BILLEASE_SHOP_CODE',
                    'size' => 40,
                    'required' => true,
                ],
                [
                    'type' => 'textarea',
                    'label' => $this->l('Merchant JWT'),
                    'name' => 'BILLEASE_MERCHANT_JWT',
                    'cols' => 80,
                    'required' => true,
                ],
                [
                    'type' => 'switch',
                    'label' => $this->l('Test mode'),
                    'name' => 'BILLEASE_TEST_MODE',
                    'values' => [
                        [
                            'id' => 'active_on',
                            'value' => 1,
                            'label' => $this->l('Yes'),
                        ],
                        [
                            'id' => 'active_off',
                            'value' => 0,
                            'label' => $this->l('No'),
                        ],
                    ],
                    'required' => true,
                ],
                [
                    'type' => 'textarea',
                    'label' => $this->l('Payment method description'),
                    'name' => 'BILLEASE_PAYMENT_METHOD_DESCRIPTION',
                    'cols' => 80,
                    'required' => false,
                ],
            ],
            'submit' => [
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right',
            ],
        ];

        $index = AdminController::$currentIndex;

        $helper = new HelperForm();

        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = $index . '&configure=' . $this->name;

        $helper->default_form_language = $defaultLang;
        $helper->allow_employee_form_lang = $defaultLang;

        $helper->title = $this->displayName;
        $helper->show_toolbar = true;
        $helper->toolbar_scroll = true;
        $helper->submit_action = 'submit' . $this->name;

        $helper->toolbar_btn = [
            'save' => [
                'desc' => $this->l('Save'),
                'href' => $index . '&configure=' . $this->name . '&save' . $this->name .
                    '&token=' . Tools::getAdminTokenLite('AdminModules'),
            ],
            'back' => [
                'href' => $index . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list'),
            ],
        ];

        $payment_method_description = Configuration::get('BILLEASE_PAYMENT_METHOD_DESCRIPTION');

        if (!$payment_method_description) {
            $payment_method_description = 'Split your payments in 3 equal installments or configure terms up to 12 months using this payment method. You can instantly sign up or login at checkout to complete your purchase.';
        }

        $helper->fields_value['BILLEASE_MERCHANT_CODE'] = Configuration::get('BILLEASE_MERCHANT_CODE');
        $helper->fields_value['BILLEASE_SHOP_CODE'] = Configuration::get('BILLEASE_SHOP_CODE');
        $helper->fields_value['BILLEASE_MERCHANT_JWT'] = Configuration::get('BILLEASE_MERCHANT_JWT');
        $helper->fields_value['BILLEASE_TEST_MODE'] = Configuration::get('BILLEASE_TEST_MODE');
        $helper->fields_value['BILLEASE_PAYMENT_METHOD_DESCRIPTION'] = $payment_method_description;

        return $helper->generateForm($fieldsForm);
    }

    protected function createOrderState()
    {
        if (!Configuration::get('BILLEASE_OS_NEW')) {
            $state = new OrderState();
            $state->name = [];

            foreach (Language::getLanguages() as $language) {
                $state->name[$language['id_lang']] = 'Awaiting BillEase payment';
            }

            $state->send_email = false;
            $state->color = '#017FBA';
            $state->hidden = false;
            $state->delivery = false;
            $state->logable = false;
            $state->invoice = false;
            $state->module_name = $this->name;

            if ($state->add()) {
                Configuration::updateValue('BILLEASE_OS_NEW', (int)$state->id);
            }
        }

        return true;
    }

    public function getPathUri()
    {
        return parent::getPathUri();
    }

    public function getOrderById($id)
    {
        if (empty($id)) {
            return;
        }

        $order = new Order($id);
        if (Validate::isLoadedObject($order)) {
            return new BillEaseOrder($order);
        }
    }

    public function getCurrentOrder()
    {
        if (empty($this->currentOrder)) {
            return;
        }

        return $this->getOrderById($this->currentOrder);
    }
}
