<?php

/**
 * @property BillEase $module
 */
class BillEaseSubmitModuleFrontController extends ModuleFrontController
{
    public function postProcess()
    {
        $cart = $this->context->cart;

        if ($cart->id_customer == 0
            || $cart->id_address_delivery == 0
            || $cart->id_address_invoice == 0
            || !$this->module->active
        ) {
            BillEaseHelper::redirectBackToCart();
        }

        $customer = new Customer($cart->id_customer);

        if (!Validate::isLoadedObject($customer)) {
            BillEaseHelper::redirectBackToCart();
        }

        $currency = $this->context->currency;

        $cart_summary = $cart->getSummaryDetails();

        $total = (float)$cart_summary['total_price'];

        $this->module->validateOrder(
            (int)$cart->id,
            Configuration::get('BILLEASE_OS_NEW'),
            $total,
            $this->module->displayName,
            null,
            null,
            (int)$currency->id,
            false,
            $customer->secure_key
        );

        $order_id = $this->module->currentOrder;

        $url_redirect = $this->getCreateQuoteRedirectForCurrentOrder(
            $cart->id,
            $this->module->id,
            $order_id,
            $customer->secure_key
        );

        $shipping_address = $cart_summary['delivery'];
        $billing_address = $cart_summary['invoice'];

        $items = [];
        $cart_items = $cart_summary['products'];

        foreach ($cart_items as $cart_item) {
            $product = new Product(
                $cart_item['id_product'],
                false,
                $this->context->language->id
            );

            $items[] = [
                'code' => $cart_item['reference'],
                'item' => $cart_item['name'],
                'price' => $cart_item['price_with_reduction'],
                'quantity' => $cart_item['quantity'],
                'currency' => $currency->iso_code,
                'url_item' => $this->context->link->getProductLink($product->id),
                'url_img' => $this->context->link->getImageLink(
                    $product->link_rewrite,
                    $product->getCover($product->id)['id_image'],
                    'home_default'
                ),
                'category' => $cart_item['category'],
                'item_type' => 'item'
            ];
        }

        $total_shipping = $cart_summary['total_shipping'];

        if (!!$total_shipping) {
            $items[] = [
                'code' => 'shipping_fee',
                'item' => 'Shipping Fee',
                'price' => $total_shipping,
                'quantity' => 1,
                'currency' => $currency->iso_code,
                'item_type' => 'fee'
            ];
        }

        $total_discounts = $cart_summary['total_discounts'];

        if (!!$total_discounts) {
            $items[] = [
                'code' => 'discount',
                'item' => 'Discount',
                'price' => -$total_discounts,
                'quantity' => 1,
                'currency' => $currency->iso_code,
                'item_type' => 'fee'
            ];
        }

        $response = BillEaseAPI::checkout([
            'merchant_code' => \BillEase\Settings::$merchantCode,
            'shop_code' => \BillEase\Settings::$shopCode,
            'amount' => $total,
            'currency' => $currency->iso_code,
            'checkout_type' => 'standard',
            'is_async' => false,
            'order_id' => (new Order($order_id))->reference,
            'url_redirect' => $url_redirect,
            'callbackapi_url' => _PS_BASE_URL_SSL_ . __PS_BASE_URI__ . 'api/orders/' . $order_id,
            'customer' => [
                'first_name' => $customer->firstname,
                'last_name' => $customer->lastname,
                'email' => $customer->email,
                'phone' => $shipping_address->phone_mobile,
                'full_name' => $customer->firstname . ' ' . $customer->lastname,
                'internal_user_id' => $customer->id,
                'adr_shipping' => [
                    'addr_type' => 'shipping',
                    'country' => Country::getIsoById($shipping_address->id_country),
                    'province' => isset($shipping_address->province) ? $shipping_address->province : null,
                    'city' => $shipping_address->city,
                    'barangay' => isset($shipping_address->barangay) ? $shipping_address->barangay : null,
                    'street' => isset($shipping_address->street) ? $shipping_address->street : null,
                    'address' => $shipping_address->address1,
                ],
                'adr_billing' => [
                    'addr_type' => 'billing',
                    'country' => Country::getIsoById($billing_address->id_country),
                    'province' => isset($billing_address->province) ? $billing_address->province : null,
                    'city' => $billing_address->city,
                    'barangay' => isset($billing_address->barangay) ? $billing_address->barangay : null,
                    'street' => isset($billing_address->street) ? $billing_address->street : null,
                    'address' => $billing_address->address1,
                ]
            ],
            'items' => $items
        ]);

        if ($response['status'] != 200) {
            $message = 'Failed to create transaction.';

            if (isset($response['data']['message'])) {
                $message = $message . ' ' . $response['data']['message'];
            }

            if (isset($response['data']['data'])) {
                $message = $message . ' Data: ' . json_encode($response['data']['data']);
            }

            throw new Exception($message);
        }

        BillEaseHelper::redirect($response['data']['redirect_url']);
    }

    protected function getCreateQuoteRedirectForCurrentOrder($cartId, $moduleId, $orderId, $customerKey)
    {
        return (
            _PS_BASE_URL_SSL_ . __PS_BASE_URI__
            . 'order-confirmation?id_cart=' . $cartId
            . '&id_module=' . $moduleId
            . '&id_order=' . $orderId
            . '&key=' . $customerKey
        );
    }
}
